package cn.com.qmth.print.manage.controller;

import cn.com.qmth.print.manage.entity.ExamEntity;
import cn.com.qmth.print.manage.entity.UserEntity;
import cn.com.qmth.print.manage.enums.RoleMeta;
import cn.com.qmth.print.manage.service.AuthService;
import cn.com.qmth.print.manage.service.ExamService;
import cn.com.qmth.print.manage.service.PmSession;
import cn.com.qmth.print.manage.service.UserService;
import cn.com.qmth.print.manage.vo.CommonQueryVo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Date: 2021/11/17.
 */
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX)
public class SysController extends BaseController {

    @Resource
    private AuthService authService;
    @Resource
    private UserService userService;
    @Resource
    private ExamService examService;

    /**
     * 登录
     *
     * @param loginName
     * @param password
     * @return
     */
    @RequestMapping(value = "/login", method = RequestMethod.POST)
    public Object login(@RequestParam String loginName, @RequestParam String password) {
        return authService.login(loginName, password);
    }

    @Aac(strict = false, auth = true)
    @PostMapping("/logout")
    public void logout(@RequestAttribute PmSession accessEntity) {
        authService.logout(accessEntity);
    }

    /**
     * 公共接口-查询用户数据
     * <p>
     * 1.管理员-用户管理 : 根据所选角色查询,是否启用根据选择。
     * 2.印点负责人-用户管理 : 只查询校验员角色,只查询自己创建的,是否启用根据选择。
     * 3.管理员-批次管理 : 只查询印点负责人角色,是否启用根据选择
     * </p>
     *
     * @param accessEntity 当前登录人信息
     * @param role         角色
     * @param creatorDpr   是否启用当前人为用户创建人权限
     * @return 用户集合
     */
    @Aac(strict = false, auth = true)
    @ApiOperation(value = "公共接口-查询用户数据")
    @RequestMapping(value = "/common/find_user_list", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = CommonQueryVo.class)})
    public Object findUserList(@RequestAttribute PmSession accessEntity,
                               @ApiParam(value = "角色") @RequestParam(required = false) RoleMeta role,
                               @ApiParam(value = "当前登录人创建") @RequestParam(required = false) Boolean creatorDpr,
                               @ApiParam(value = "是否启用？") @RequestParam(required = false) Boolean enable) {


        QueryWrapper<UserEntity> queryWrapper = new QueryWrapper<>();

        if (Objects.nonNull(role)) {
            queryWrapper.lambda().eq(UserEntity::getRole, role);
        }
        if (Objects.nonNull(creatorDpr) && creatorDpr) {
            Long userId = getAccessUserId(accessEntity);
            queryWrapper.lambda().eq(UserEntity::getCreatorId, userId);
        }
        if (Objects.nonNull(enable)) {
            queryWrapper.lambda().eq(UserEntity::isEnable, enable);
        }

        return userService.list(queryWrapper).stream().flatMap(e -> {
            CommonQueryVo vo = new CommonQueryVo();
            vo.setId(e.getId());
            vo.setValue(e.getName());
            return Stream.of(vo);
        }).collect(Collectors.toList());
    }

    /**
     * 公共接口-查询考试批次数据
     * <p>
     * 1.管理员-批次管理 : 根据所选印点负责人查询
     * 2.印点负责人-批次管理 : 根据当前登录人id作为印点负责人id查询
     * </p>
     *
     * @param userId 印点负责人id
     * @return 考试批次集合
     */
    @Aac(strict = false, auth = true)
    @ApiOperation(value = "公共接口-查询考试批次数据")
    @RequestMapping(value = "/common/find_exam_list", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = CommonQueryVo.class)})
    public Object findExamList(@ApiParam(value = "印点负责人Id", required = true) @RequestParam Long userId) {
        return examService.list(new QueryWrapper<ExamEntity>().lambda().eq(ExamEntity::getCreatorId, userId))
                .stream().flatMap(e -> {
                    CommonQueryVo vo = new CommonQueryVo();
                    vo.setId(e.getId());
                    vo.setValue(e.getName());
                    return Stream.of(vo);
                }).collect(Collectors.toList());
    }
}
