package cn.com.qmth.print.manage.controller;

import cn.com.qmth.print.manage.config.PmConstants;
import cn.com.qmth.print.manage.entity.UserEntity;
import cn.com.qmth.print.manage.enums.RoleMeta;
import cn.com.qmth.print.manage.service.PmSession;
import cn.com.qmth.print.manage.service.UserService;
import cn.com.qmth.print.manage.service.query.UserQuery;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.exception.StatusException;
import io.swagger.annotations.ApiOperation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Date;
import java.util.Objects;

/**
 * @Date: 2021/11/16.
 */
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/user")
@Aac(strict = false, auth = true)
public class UserController extends BaseController {

    @Resource
    private UserService userService;

    /**
     * 用户分页查询
     *
     * @param query
     * @return
     */
    @RequestMapping(value = "/page", method = RequestMethod.POST)
    public Object page(@RequestAttribute PmSession accessEntity, @Validated UserQuery query) {
        Long userId = getAccessUserId(accessEntity);
        UserEntity requestUser = userService.getById(userId);
        Long printUserId = null;
        if (RoleMeta.PRINT_LEADER.equals(requestUser.getRole())) {
            printUserId = userId;
        }
        return userService.pageQuery(printUserId, query);
    }

    /**
     * 用户查询
     *
     * @return
     */
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Object list() {
        return userService.listByEnable();
    }

    /**
     * 新增/修改用户
     *
     * @return
     */
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Object save(@RequestAttribute PmSession accessEntity,
                       @RequestParam Long id,
                       @RequestParam String loginName,
                       @RequestParam String name,
                       @RequestParam String password,
                       @RequestParam boolean enable,
                       @RequestParam RoleMeta role,
                       @RequestParam(required = false) MultipartFile file) throws IOException {
        Long userId = getAccessUserId(accessEntity);
        UserEntity requestUser = userService.getById(userId);
        if (Objects.isNull(requestUser)) {
            throw new StatusException("当前用户不存在");
        }
        return userService.save(requestUser, id, loginName, name, role, password, enable, file);
    }

    @ApiOperation(value = "修改用户密码", notes = "修改密码")
    @RequestMapping(value = "/update_password", method = RequestMethod.POST)
    @Transactional
    public void updatePassword(@RequestAttribute PmSession accessEntity, @RequestParam String password) {
        Long userId = getAccessUserId(accessEntity);
        UserEntity requestUser = userService.getById(userId);
        requestUser.setPassword(password);
        requestUser.setPasswordCount(requestUser.getPasswordCount() + 1);
        requestUser.setUpdaterId(userId);
        requestUser.setUpdateTime(new Date());
        userService.updateById(requestUser);
    }


    @ApiOperation(value = "重置用户密码", notes = "重置密码")
    @RequestMapping(value = "/reset_password", method = RequestMethod.POST)
    @Transactional
    public void resetPassword(@RequestAttribute PmSession accessEntity, @RequestParam Long userId) {
        Long requestUserId = getAccessUserId(accessEntity);
        UserEntity requestUser = userService.getById(requestUserId);

        if (!RoleMeta.ADMIN.equals(requestUser.getRole())) {
            throw new StatusException("只有管理员能重置密码");
        }
        UpdateWrapper<UserEntity> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda()
                .eq(UserEntity::getId, userId)
                .set(UserEntity::getPassword, PmConstants.DEFAULT_PASSWORD)
                .set(UserEntity::getPasswordCount, 0)
                .set(UserEntity::getUpdaterId, userId)
                .set(UserEntity::getUpdateTime, new Date());
        userService.update(updateWrapper);
    }
}